@@ -1,4 +1,4 @@
-/* $Id: thunar-history.c 22934 2006-08-29 15:04:52Z benny $ */
+/* $Id$ */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
  *
@@ -40,9 +40,7 @@ enum
 
 
 
-static void         thunar_history_class_init             (ThunarHistoryClass   *klass);
 static void         thunar_history_navigator_init         (ThunarNavigatorIface *iface);
-static void         thunar_history_init                   (ThunarHistory        *history);
 static void         thunar_history_dispose                (GObject              *object);
 static void         thunar_history_finalize               (GObject              *object);
 static void         thunar_history_get_property           (GObject              *object,
@@ -96,44 +94,8 @@ struct _ThunarHistory
 
 
 
-static GObjectClass *thunar_history_parent_class;
-
-
-
-GType
-thunar_history_get_type (void)
-{
-  static GType type = G_TYPE_INVALID;
-
-  if (G_UNLIKELY (type == G_TYPE_INVALID))
-    {
-      static const GTypeInfo info =
-      {
-        sizeof (ThunarHistoryClass),
-        NULL,
-        NULL,
-        (GClassInitFunc) thunar_history_class_init,
-        NULL,
-        NULL,
-        sizeof (ThunarHistory),
-        0,
-        (GInstanceInitFunc) thunar_history_init,
-        NULL,
-      };
-
-      static const GInterfaceInfo navigator_info =
-      {
-        (GInterfaceInitFunc) thunar_history_navigator_init,
-        NULL,
-        NULL,
-      };
-
-      type = g_type_register_static (G_TYPE_OBJECT, I_("ThunarHistory"), &info, 0);
-      g_type_add_interface_static (type, THUNAR_TYPE_NAVIGATOR, &navigator_info);
-    }
-
-  return type;
-}
+G_DEFINE_TYPE_WITH_CODE (ThunarHistory, thunar_history, G_TYPE_OBJECT,
+    G_IMPLEMENT_INTERFACE (THUNAR_TYPE_NAVIGATOR, thunar_history_navigator_init))
 
 
 
@@ -142,9 +104,6 @@ thunar_history_class_init (ThunarHistoryClass *klass)
 {
   GObjectClass *gobject_class;
 
-  /* determine the parent type class */
-  thunar_history_parent_class = g_type_class_peek_parent (klass);
-
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->dispose = thunar_history_dispose;
   gobject_class->finalize = thunar_history_finalize;
@@ -312,20 +271,11 @@ thunar_history_set_current_directory (ThunarNavigator *navigator,
   if (G_UNLIKELY (current_directory == history->current_directory))
     return;
 
-  /* we try to be smart and check if the new current directory
-   * is the first element on either "back" or "forward" and if
-   * so, perform the appropriate operation.
-   */
-  if (history->back_list != NULL && history->back_list->data == current_directory)
-    {
-      /* go back one step */
-      thunar_history_go_back (history, 1);
-    }
-  else if (history->forward_list != NULL && history->forward_list->data == current_directory)
-    {
-      /* go forward one step */
-      thunar_history_go_forward (history, 1);
-    }
+  /* if the new directory is the first one in the forward history, we
+   * just move forward one step instead of clearing the whole forward
+   * history */
+  if (history->forward_list != NULL && history->forward_list->data == current_directory)
+    thunar_history_go_forward (history, 1);
   else
     {
       /* clear the "forward" list */
